from decimal import Decimal
from typing import Any, Dict, List

from .subapi import CimSubAPI


class OrdersAPI(CimSubAPI):
    """
    订单相关接口：dropship 报价、创建、分页、详情、取消
    """

    # ---- 报价 ----

    def get_total_price(self, payload: Dict[str, Any]) -> Dict[str, Any]:
        """
        /dropship/order/total/price
        payload 结构参考你之前抓到的 curl
        """
        return self._request(
            "POST",
            "/dropship/order/total/price",
            json=payload,
            headers={"content-type": "application/json", "origin": "https://cim.cameronsino.com"},
        )

    @staticmethod
    def pick_cheapest_express(express_list: List[Dict[str, Any]]) -> Dict[str, Any]:
        """
        根据 price + dgFee 选出最便宜的一条：
        [
          {"expressName": "USPS","price": "6.28","dgFee": "0.00","currency": "USD"},
          ...
        ]
        """
        if not express_list:
            raise ValueError("express_list 不能为空")

        def total_cost(x: Dict[str, Any]) -> Decimal:
            return Decimal(str(x.get("price") or 0)) + Decimal(str(x.get("dgFee") or 0))

        return min(express_list, key=total_cost)

    # ---- 创建 Dropship 订单 ----

    def create(self, payload: Dict[str, Any]) -> Dict[str, Any]:
        """
        /dropship/create
        """
        return self._request(
            "POST",
            "/dropship/create",
            json=payload,
            headers={"content-type": "application/json", "origin": "https://cim.cameronsino.com"},
        )

    # ---- 分页查询 ----

    def list_page(
        self,
        page_no: int = 1,
        page_size: int = 20,
        *,
        date: str = "",
        start_date: str = "",
        end_date: str = "",
        item_no: str = "",
        order_status: str = "",
        search: str = "",
        shipment_region: str = "",
        type_: str = "",
    ) -> Dict[str, Any]:
        """
        /dropship/page
        """
        params = {
            "pageNo": str(page_no),
            "pageSize": str(page_size),
            "date": date,
            "startDate": start_date,
            "endDate": end_date,
            "itemNo": item_no,
            "orderStatus": order_status,
            "search": search,
            "shipmentRegion": shipment_region,
            "type": type_,
        }
        return self._request("GET", "/dropship/page", params=params)

    # ---- 详情 ----

    def get_detail(self, dropship_id: str) -> Dict[str, Any]:
        """
        /dropship/detail?dropShipId=US-xxxx
        """
        params = {"dropShipId": dropship_id}
        return self._request("GET", "/dropship/detail", params=params)

    # ---- 取消 ----

    def cancel(self, dropship_id: str) -> Dict[str, Any]:
        """
        对于 pending状态的订单取消
        /dropship/cancel?dropshipId=US-xxxx  （PUT）
        """
        params = {"dropshipId": dropship_id}
        return self._request(
            "PUT",
            "/dropship/cancel",
            params=params,
            headers={"origin": "https://cim.cameronsino.com"},
        )

    def cancel_approval(self, dropship_id: str) -> Dict[str, Any]:
        """
        对于 Processing状态的订单取消
        /dropship/cancel?dropshipId=US-xxxx  （PUT）
        """
        params = {"dropshipId": dropship_id}
        return self._request(
            "PUT",
            "/dropship/cancel",
            params=params,
            headers={"origin": "https://cim.cameronsino.com"},
        )



from dataclasses import dataclass


@dataclass
class CIMConfig:
    """
    CIM 客户端配置：
    - base_url: API 前缀
    - tenant_id: 现在你这边都是 1
    - timeout: 请求超时时间
    - user_agent: SDK 自定义 UA
    """

    base_url: str = "https://cim.cameronsino.com/admin-api/cim"
    tenant_id: str = "1"
    user_agent: str = "Mozilla/5.0 (CIM-SDK-Python)"
    timeout: int = 20



import datetime

from cim_sdk.orders import OrdersAPI
from cim_sdk.transactions import TransactionsAPI


def test_pick_cheapest_express_selects_min_total_cost():
    express_list = [
        {"expressName": "A", "price": "10.00", "dgFee": "1.00"},
        {"expressName": "B", "price": "6.00", "dgFee": "0.50"},
        {"expressName": "C", "price": "5.50", "dgFee": "2.00"},
    ]

    result = OrdersAPI.pick_cheapest_express(express_list)

    assert result["expressName"] == "B"


def test_get_recent_records_builds_expected_params():
    today = datetime.date.today()
    start = today - datetime.timedelta(days=2)

    class DummyClient:
        def __init__(self):
            self.last_request = None

        def request(self, method, path, params=None, json=None, headers=None):
            self.last_request = {
                "method": method,
                "path": path,
                "params": params,
            }
            return {"ok": True}

    dummy_client = DummyClient()
    api = TransactionsAPI(dummy_client)
    api.get_recent_records(past_days=2, page_size=50, keyword="foo")

    params = dummy_client.last_request["params"]
    assert dummy_client.last_request["path"] == "/transaction-record"
    assert params["startDate"] == start.strftime("%Y-%m-%d")
    assert params["endDate"] == today.strftime("%Y-%m-%d")
    assert params["pageSize"] == "50"
    assert params["keyWord"] == "foo"



"""CameronSino CIM Admin API 的 Python SDK 包入口。"""

from .client import CIMClient
from .config import CIMConfig
from .http import CIMApiError
from .token_manager import get_cim_client_for_username

__all__ = ["CIMClient", "CIMConfig", "CIMApiError", "get_cim_client_for_username"]



from typing import Any, Dict, Optional

from .http import BaseHttpClient


class CimSubAPI:
    """
    所有领域子 API 的基类，内部持有一个 CIMClient / BaseHttpClient，
    提供 _request() 便于复用底层 HTTP 能力。
    """

    def __init__(self, client: BaseHttpClient):
        self.client = client

    def _request(
        self,
        method: str,
        path: str,
        *,
        params: Optional[Dict[str, Any]] = None,
        json: Optional[Dict[str, Any]] = None,
        headers: Optional[Dict[str, str]] = None,
    ) -> Dict[str, Any]:
        return self.client.request(method, path, params=params, json=json, headers=headers)



import datetime
from typing import Any, Dict

from .subapi import CimSubAPI


class TransactionsAPI(CimSubAPI):
    """
    财务流水 / 交易记录相关接口
    """

    def get_records(
        self,
        start_date: str,
        end_date: str,
        *,
        page_no: int = 1,
        page_size: int = 500,
        type_: str = "ALL",
        keyword: str = "",
    ) -> Dict[str, Any]:
        """
        /transaction-record
        """
        params = {
            "startDate": start_date,
            "endDate": end_date,
            "pageNo": str(page_no),
            "pageSize": str(page_size),
            "type": type_,
            "keyWord": keyword,
        }
        return self._request("GET", "/transaction-record", params=params)

    def get_recent_records(self, past_days: int = 3, **kwargs: Any) -> Dict[str, Any]:
        """
        获取最近 N 天的流水（简单封装）
        """
        today = datetime.date.today()
        start = today - datetime.timedelta(days=past_days)
        return self.get_records(
            start_date=start.strftime("%Y-%m-%d"),
            end_date=today.strftime("%Y-%m-%d"),
            **kwargs,
        )



from datetime import datetime
from typing import TYPE_CHECKING, Any, Dict, Optional

from .config import CIMConfig
from .http import BaseHttpClient, CIMApiError

if TYPE_CHECKING:  # 避免循环导入
    from .orders import OrdersAPI
    from .products import ProductsAPI
    from .transactions import TransactionsAPI


class CIMClient(BaseHttpClient):
    """
    根客户端：
    - 负责 token / config / session / 基础 request
    - 暴露领域子 API：orders / transactions / products（懒加载）
    """

    def __init__(self, *, token: Optional[str] = None, config: Optional[CIMConfig] = None):
        super().__init__(config=config)
        if token:
            self.set_token(token)

        self.expires_time: Optional[datetime] = None
        self.refresh_token: Optional[str] = None
        self.user_id: Optional[str] = None
        self.has_order: Optional[bool] = None

        # 子模块懒加载占位
        self._orders: Optional["OrdersAPI"] = None
        self._transactions: Optional["TransactionsAPI"] = None
        self._products: Optional["ProductsAPI"] = None

    def login(self, username: str, password: str, uuid: str = "") -> Dict[str, Any]:
        """
        调用 /auth/login 接口，并把 accessToken 写入 client.token。

        返回 data 字段的内容，形如：
        {
            "userId": "...",
            "accessToken": "...",
            "refreshToken": "...",
            "expiresTime": "2025-12-04 08:34:41",
            "hasOrder": true
        }
        """
        payload = {
            "username": username,
            "password": password,
            "uuid": uuid,
        }

        resp = self.request(
            "POST",
            "/auth/login",
            json=payload,
            headers={
                "content-type": "application/json",
                "origin": "https://cim.cameronsino.com",
            },
        )

        code = resp.get("code")
        if code != 0:
            msg = resp.get("msg") or "登录失败（未知错误）"
            raise CIMApiError(f"登录失败: code={code}, msg={msg}")

        data = resp.get("data") or {}
        access_token = data.get("accessToken")
        if not access_token:
            raise CIMApiError("登录响应中缺少 accessToken")

        self.set_token(access_token)

        self.user_id = data.get("userId")
        self.refresh_token = data.get("refreshToken")
        self.has_order = data.get("hasOrder")

        expires_str = data.get("expiresTime")
        if expires_str:
            try:
                self.expires_time = datetime.strptime(expires_str, "%Y-%m-%d %H:%M:%S")
            except ValueError:
                self.expires_time = None

        return data

    @classmethod
    def from_credentials(
        cls,
        username: str,
        password: str,
        uuid: str = "",
        *,
        config: Optional[CIMConfig] = None,
    ) -> "CIMClient":
        """
        简化构造：提供账号信息直接完成登录。
        """
        client = cls(config=config)
        client.login(username, password, uuid=uuid)
        return client

    # ===== 懒加载子 API =====

    @property
    def orders(self) -> "OrdersAPI":
        if self._orders is None:
            from .orders import OrdersAPI

            self._orders = OrdersAPI(self)
        return self._orders

    @property
    def transactions(self) -> "TransactionsAPI":
        if self._transactions is None:
            from .transactions import TransactionsAPI

            self._transactions = TransactionsAPI(self)
        return self._transactions

    @property
    def products(self) -> "ProductsAPI":
        if self._products is None:
            from .products import ProductsAPI

            self._products = ProductsAPI(self)
        return self._products



from typing import Any, Dict

from .subapi import CimSubAPI


class ProductsAPI(CimSubAPI):
    """
    产品相关接口（预留，将来你有 /product/... 之类的接口再慢慢加）
    """

    # 示例：将来可以加
    # def get_product(self, item_no: str) -> Dict[str, Any]:
    #     return self._request("GET", f"/product/detail/{item_no}")



"""
基于业务表 `cs_user_info` 的 Token 复用工厂，随 SDK 一起分发（pip 安装可直接导入）。

表结构（核心字段）：
    username (PK), cs_password, Rcustomerno, token, exp_time, exp_times_tamp,
    refreshToken, userId, hasOrder
"""

from __future__ import annotations

import datetime as _dt
from typing import TYPE_CHECKING, Any, Dict, Optional

from cim_sdk import CIMClient

if TYPE_CHECKING:
    from cim_sdk import CIMConfig

TABLE_NAME = "cs_user_info"
DEFAULT_SAFETY_MARGIN_SECONDS = 60


def _parse_exp_time(raw: Any) -> Optional[_dt.datetime]:
    """将 exp_time / expiresTime 字段解析为 datetime。"""
    if raw is None:
        return None
    if isinstance(raw, _dt.datetime):
        return raw
    if isinstance(raw, _dt.date):
        return _dt.datetime.combine(raw, _dt.time())
    if isinstance(raw, str):
        for fmt in ("%Y-%m-%d %H:%M:%S", "%Y-%m-%d"):
            try:
                return _dt.datetime.strptime(raw, fmt)
            except ValueError:
                continue
        try:
            return _dt.datetime.fromisoformat(raw)
        except ValueError:
            return None
    return None


def _fetch_user_record(db_conn: Any, value: str, *, column: str = "username") -> Optional[Dict[str, Any]]:
    """
    从 cs_user_info 查询用户记录，column 支持 username / Rcustomerno / userId。
    期望 db_conn 遵循 DB-API：支持 cursor() / execute() / fetchone()。
    """
    sql = f"SELECT * FROM {TABLE_NAME} WHERE {column}=%s"
    cursor = db_conn.cursor()
    try:
        cursor.execute(sql, (value,))
        row = cursor.fetchone()
        if row is None:
            return None
        if isinstance(row, dict):
            return row
        columns = [col[0] for col in cursor.description]
        return dict(zip(columns, row))
    finally:
        cursor.close()


def _is_token_valid(record: Dict[str, Any], safety_margin_seconds: int) -> bool:
    """检查记录中的 token 是否仍然有效。"""
    token = record.get("token")
    exp_time = _parse_exp_time(record.get("exp_time"))
    if not token or not exp_time:
        return False
    now = _dt.datetime.now()
    return exp_time - _dt.timedelta(seconds=safety_margin_seconds) > now


def _update_tokens(
    db_conn: Any,
    username: str,
    *,
    access_token: str,
    refresh_token: Optional[str],
    expires_at: Optional[_dt.datetime],
    user_id: Optional[str],
    has_order: Optional[bool],
) -> None:
    """将登录结果写回 cs_user_info。"""
    exp_time_str = expires_at.strftime("%Y-%m-%d %H:%M:%S") if expires_at else None
    exp_timestamp = str(int(expires_at.timestamp())) if expires_at else None
    sql = f"""
        UPDATE {TABLE_NAME}
        SET token=%s,
            exp_time=%s,
            exp_times_tamp=%s,
            refreshToken=%s,
            userId=%s,
            hasOrder=%s
        WHERE username=%s
    """
    params = (
        access_token,
        exp_time_str,
        exp_timestamp,
        refresh_token,
        user_id,
        int(has_order) if has_order is not None else None,
        username,
    )
    cursor = db_conn.cursor()
    try:
        cursor.execute(sql, params)
        db_conn.commit()
    finally:
        cursor.close()


def get_cim_client_for_username(
    db_conn: Any,
    username: Optional[str] = None,
    *,
    safety_margin_seconds: int = DEFAULT_SAFETY_MARGIN_SECONDS,
    config: Optional["CIMConfig"] = None,
    uuid: str = "",
    rcustomerno: Optional[str] = None,
    user_id: Optional[str] = None,
) -> CIMClient:
    """
    工厂方法：基于 cs_user_info 的 token 复用逻辑，返回已配置好的 CIMClient。

    步骤：
    1. 按 username 读取 cs_user_info 记录；
    2. 若 token 未过期，直接 set_token；
    3. 若 token 缺失或过期，用存储的 cs_password 自动登录并更新 token 信息。
    """
    # 兼容：优先按传入的 rcustomerno / user_id 查询，否则按 username。
    query_column = "username"
    query_value = username
    if rcustomerno:
        query_column = "Rcustomerno"
        query_value = rcustomerno
    if user_id:
        query_column = "userId"
        query_value = user_id

    if not query_value:
        raise ValueError("必须提供 username、rcustomerno 或 user_id 之一用于查询。")

    record = _fetch_user_record(db_conn, query_value, column=query_column)
    if not record:
        raise ValueError(f"未找到对应记录，查询字段 {query_column} 值 {query_value}")

    stored_username = record.get("username") or username or query_value
    stored_password = record.get("cs_password")
    if not stored_password:
        raise ValueError(f"用户 {stored_username} 缺少 cs_password，无法自动登录")

    client = CIMClient(config=config)

    if _is_token_valid(record, safety_margin_seconds):
        client.set_token(record["token"])
        expires_at = _parse_exp_time(record.get("exp_time"))
        client.expires_time = expires_at
        client.refresh_token = record.get("refreshToken")
        client.user_id = record.get("userId")
        client.has_order = record.get("hasOrder")
        return client

    login_data = client.login(stored_username, stored_password, uuid=uuid)
    expires_at = _parse_exp_time(login_data.get("expiresTime"))

    _update_tokens(
        db_conn,
        stored_username,
        access_token=login_data.get("accessToken"),
        refresh_token=login_data.get("refreshToken"),
        expires_at=expires_at,
        user_id=login_data.get("userId"),
        has_order=login_data.get("hasOrder"),
    )
    return client



from typing import Any, Dict, Optional

import requests

from .config import CIMConfig


class CIMApiError(Exception):
    """调用 CIM 接口失败时抛出的异常"""

    def __init__(self, message: str, status_code: Optional[int] = None, response: Optional[requests.Response] = None):
        super().__init__(message)
        self.status_code = status_code
        self.response = response


class BaseHttpClient:
    """
    负责：
    - 保存 config / session / token
    - 统一封装 HTTP 请求（带好 headers）
    """

    def __init__(self, config: Optional[CIMConfig] = None, session: Optional[requests.Session] = None):
        self.config = config or CIMConfig()
        self.session = session or requests.Session()
        self.token: Optional[str] = None

    # ========== token 相关 ==========

    def set_token(self, token: str) -> None:
        self.token = token

    # ========== headers & request ==========

    def _headers(self, extra_headers: Optional[Dict[str, str]] = None) -> Dict[str, str]:
        headers = {
            "tenant-id": self.config.tenant_id,
            "user-agent": self.config.user_agent,
        }
        if self.token:
            headers["authorization"] = f"Bearer {self.token}"
        if extra_headers:
            headers.update(extra_headers)
        return headers

    def request(
        self,
        method: str,
        path: str,
        *,
        params: Optional[Dict[str, Any]] = None,
        json: Optional[Dict[str, Any]] = None,
        headers: Optional[Dict[str, str]] = None,
    ) -> Dict[str, Any]:
        """
        统一的请求入口，返回 JSON dict，失败抛 CIMApiError
        """
        url = f"{self.config.base_url.rstrip('/')}/{path.lstrip('/')}"
        merged_headers = self._headers(headers)

        try:
            resp = self.session.request(
                method=method.upper(),
                url=url,
                params=params,
                json=json,
                headers=merged_headers,
                timeout=self.config.timeout,
            )
        except requests.RequestException as exc:
            raise CIMApiError(f"网络请求异常: {exc}") from exc

        if not resp.ok:
            try:
                err_json = resp.json()
            except Exception:
                err_json = None

            msg = f"CIM API 调用失败: HTTP {resp.status_code}"
            if isinstance(err_json, dict):
                msg_detail = err_json.get("msg") or err_json.get("message")
                if msg_detail:
                    msg += f" - {msg_detail}"
            raise CIMApiError(msg, status_code=resp.status_code, response=resp)

        try:
            return resp.json()
        except ValueError as exc:
            raise CIMApiError("响应不是合法 JSON", status_code=resp.status_code, response=resp) from exc



import datetime

from cim_sdk.token_manager import get_cim_client_for_username


class _FakeCursor:
    def __init__(self, future_seconds: int = 3600):
        self.executed_sql = None
        self.executed_params = None
        self.future = future_seconds

    def execute(self, sql, params):
        self.executed_sql = sql
        self.executed_params = params

    def fetchone(self):
        # 返回 dict，避免依赖 description
        exp_time = (datetime.datetime.now() + datetime.timedelta(seconds=self.future)).strftime("%Y-%m-%d %H:%M:%S")
        return {
            "username": "user-1",
            "cs_password": "pwd",
            "Rcustomerno": "CUST-9",
            "token": "token-abc",
            "exp_time": exp_time,
            "exp_times_tamp": str(int(datetime.datetime.now().timestamp()) + self.future),
            "refreshToken": "r1",
            "userId": "U100",
            "hasOrder": 1,
        }

    def close(self):
        pass


class _FakeDB:
    def __init__(self, future_seconds: int = 3600):
        self.cursor_obj = _FakeCursor(future_seconds)

    def cursor(self):
        return self.cursor_obj

    def commit(self):
        pass


def _assert_query(column_name: str, value: str):
    db = _FakeDB()
    client = get_cim_client_for_username(
        db,
        username=None,
        rcustomerno=value if column_name == "Rcustomerno" else None,
        user_id=value if column_name == "userId" else None,
    )
    sql = db.cursor_obj.executed_sql.lower()
    assert column_name.lower() in sql
    assert db.cursor_obj.executed_params == (value,)
    assert client.token == "token-abc"


def test_query_by_rcustomerno():
    _assert_query("Rcustomerno", "CUST-9")


def test_query_by_user_id():
    _assert_query("userId", "U100")



from cim_sdk import CIMClient


def test_client_has_subapis():
    client = CIMClient()
    assert hasattr(client, "orders")
    assert hasattr(client, "transactions")
    assert hasattr(client, "products")
